package com.ls11.dominohelper;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import lotus.domino.NotesFactory;
import lotus.domino.NotesThread;
import lotus.domino.Session;

/**
 * Helper class that unifies the execution of code that needs access to the Domino session
 * 
 * @author Karsten Lehmann
 */
public class DominoExecution {

	/**
	 * Runs a piece of code so that it can access Notes Java API objects
	 * 
	 * @param <T> type of result
	 * @param callable code
	 * @return {@link Future} object to track the code execution and read the result
	 */
	public static <T> Future<T> run(final IDominoCallable<T> callable) {
		if (XPagesDetector.isXPagesContext()) {
			//Directly execute the code in the current thread and catch exceptions,
			//since we are running in an XPages thread
			final Throwable[] error = new Throwable[1];
			final Object[] result = new Object[1];
			try {
				result[0] = callable.call(XPagesDetector.getXspContextSession());
			} catch (Throwable t) {
				error[0] = t;
			}
			
			//create a Future object to return the result/exception to the caller
			return new Future<T>() {
				@Override
				public boolean cancel(boolean arg0) {
					return false;
				}

				@SuppressWarnings("unchecked")
				@Override
				public T get() throws InterruptedException, ExecutionException {
					if (error[0] != null)
						throw new ExecutionException(
								"An error occurred in the execution of Domino code:"
										+ error[0], error[0]);
					return (T) result[0];
				}

				@SuppressWarnings("unchecked")
				@Override
				public T get(long arg0, TimeUnit arg1)
						throws InterruptedException, ExecutionException,
						TimeoutException {
					if (error[0] != null)
						throw new ExecutionException(
								"An error occurred in the execution of Domino code:"
										+ error[0], error[0]);
					return (T) result[0];
				}

				@Override
				public boolean isCancelled() {
					return false;
				}

				@Override
				public boolean isDone() {
					return true;
				}
			};			
		}
		else {
			//execute the code asynchronously in a separate thread; the FutureTask takes
			//care of waiting for the code to execute when somebody calls its get() method
			FutureTask<T> task=new FutureTask<T>(new Callable<T>() {
				@Override
				public T call() throws Exception {
					Session session=NotesFactory.createSession();
					try {
						return callable.call(session);
					}
					finally {
						session.recycle();
					}
				}
			});
			NotesThread t=new NotesThread(task, "Domino code execution");
			t.start();
			return task;
		}
	}
}
